@@ -30,7 +30,7 @@
 #   include <Winsock2.h>
 #   include <process.h>
 #   include <mmsystem.h>
-#   include "dcap_win32.h"
+#   include "dcap_unix2win.h"
 #else
 #   include <sys/time.h>
 #   include <sys/times.h>
@@ -44,12 +44,16 @@
 
 #include "system_io.h"
 
+#include "dcap.h"
 #include "dcap_debug.h"
 #include "dcap_error.h"
 #include "dcap_types.h"
 #include "dcap_protocol.h"
 #include "dcap_poll.h"
 #include "dcap_mqueue.h"
+#include "dcap_str_util.h"
+#include "dcap_reconnect.h"
+#include "dcap_functions.h"
 #include "array.h"
 #include "pnfs.h"
 #include "io.h"
@@ -57,12 +61,13 @@
 #include "links.h"
 #include "dcap_url.h"
 #include "dcap_accept.h"
-#include "dcap_ahead.h"
 #include "socket_nio.h"
 #include "dcap_reconnect.h"
 #include "ioTunnel.h"
 #include "tunnelManager.h"
+#include "lineparser.h"
 #include "xutil.h"
+#include "node_plays.h"
 
 
 static char    *hostName;
@@ -95,7 +100,6 @@ static long openTimeOut = -1;
 #define PNFS_DC_LOCK "/.(config)(dCache)/dcap.LOCK"
 #define DC_LOCK_TIME 60 /* time to sleep if dcap.LOCK file exist */
 #define DC_STAGE (O_RDONLY | O_NONBLOCK)
-#include "dcap_nodes.h"
 #ifndef MAXHOSTNAMELEN
 #define		MAXHOSTNAMELEN  64
 #endif /* MAXHOSTNAMELEN */
@@ -103,39 +107,21 @@ static long openTimeOut = -1;
 
 static int activeClient = 0;
 
-/* funcions prototypes */
-
-int             cache_open(struct vsp_node *);
-int             initControlLine(struct vsp_node *);
-int             serverConnect(struct vsp_node *);
-server         *parseConfig(const char *);
-int             cache_connect(server *);
-int             sayHello(int, ioTunnel *);
-int             create_data_socket(int *, unsigned short *);
-int             ascii_open_conversation(struct vsp_node *);
-int             close_data_socket(int);
-int             sendControlMessage(int, const char *, size_t, ioTunnel *);
-asciiMessage   *getControlMessage(int, struct vsp_node *);
-int             getDataMessage(struct vsp_node *);
-int             data_hello_conversation(struct vsp_node *);
-int             sendDataMessage(struct vsp_node *, char *, int , int , ConfirmationBlock *);
-int             reconnected(struct vsp_node *, int, int64_t);
-int             dc_set_pos(struct vsp_node *, int, int64_t);
-int             newControlLine(struct vsp_node *);
-void            dc_setCallbackPortRange( unsigned short, unsigned short );
-void            dc_setCallbackPort( unsigned short );
-void            getRevision( revision * );
-void            dc_setTCPSendBuffer( int );
-void            dc_setTCPReceiveBuffer( int );
-int             init_hostname();
-
-static void     getPortRange();
+/* Local funcion prototypes */
+static int initControlLine(struct vsp_node *);
+static int serverConnect(struct vsp_node *);
+static server *parseConfig(const char *);
+static int cache_connect(server *);
+static int sayHello(int, ioTunnel *);
+static int create_data_socket(int *, unsigned short *);
+static int ascii_open_conversation(struct vsp_node *);
+static int getDataMessage(struct vsp_node *);
+static void getRevision( revision * );
+static int init_hostname();
+static void getPortRange();
 static int isActive();
-
-int get_data( struct vsp_node *);
-int get_fin( struct vsp_node *);
-int get_ack(int , ConfirmationBlock * );
-ConfirmationBlock get_reply(int);
+static ConfirmationBlock get_reply(int);
+static int name_invalid(char *);
 
 #ifndef HAVE_HTONLL
     uint64_t  htonll(uint64_t);
@@ -153,6 +139,24 @@ static MUTEX(acceptLock);
 
 /* Utilities functions */
 
+int name_invalid(char *p)
+{
+       /* Check input strings for characters which will cause protocol problems.  Currently
+       * newlines and double-quotes are problematic */
+       if (!p)
+               return 1;
+       for (; *p; ++p) {
+               switch( *p ){
+                       case '\n':
+                       case '"':
+                               return 1;
+               }
+       }
+       return 0;
+}
+
+
+
 int init_hostname()
 {
     struct hostent *he;
@@ -538,7 +542,7 @@ serverConnect(struct vsp_node * node)
 		lockMember();
 		if ((node->fd = getMember(dcache_host)) != -1) {
 
-			srv = parseConfig(node->url == NULL ? dcache_host : url2config(node->url, buffer) );
+			srv = parseConfig(node->url == NULL ? dcache_host : url2config(buffer, sizeof(buffer), node->url) );
 			if (srv == NULL) {
 				unlockMember();
 				return -1;
@@ -556,7 +560,7 @@ serverConnect(struct vsp_node * node)
 
 		newQueue(0);
 		dc_debug(DC_INFO, "Creating a new control connection to %s.",dcache_host );
-		srv = parseConfig(node->url == NULL ? dcache_host : url2config(node->url, buffer) );
+		srv = parseConfig(node->url == NULL ? dcache_host : url2config(buffer, sizeof(buffer), node->url) );
 
 		if (srv == NULL) {
 			unlockMember();
@@ -581,7 +585,6 @@ serverConnect(struct vsp_node * node)
 	}
 
 }
-extern char ** lineParser( const char *, const char *);
 
 /* current config line format:
     [tunnelType@]host[:port[:tunnelProvider[:tunnelType] ] ]
@@ -864,19 +867,20 @@ create_data_socket(int *dataFd, unsigned short *cbPort)
 
 }
 
+#define APPEND_TO_OPENSTR(STRING) dc_safe_strncat(openStr, DCAP_CMD_SIZE, STRING)
+
 int
 ascii_open_conversation(struct vsp_node * node)
 {
 
-	char           *openStr;
 	int             len;
 	int             uid;
 	short           invalid_flag = 1;
 	asciiMessage   *aM;
-
-	openStr = (char *) calloc(DCAP_CMD_SIZE, sizeof(char));
-
-	if (openStr == NULL) {
+	char_buf_t    *context;
+	char           *outStr;
+	context = dc_char_buf_create();
+	if (context == NULL) {
 		dc_errno = DEMALLOC;
 		return -1;
 	}
@@ -886,40 +890,78 @@ ascii_open_conversation(struct vsp_node * node)
 #else
 	uid = (int)getuid();
 #endif
-
-	sprintf(openStr, "%d 0 client %s \"%s\"", node->queueID,
+	outStr = dc_char_buf_sprintf(context,"%d 0 client %s \"%s\"", node->queueID,
 	                                          asciiCommand(node->asciiCommand),
-											  node->asciiCommand == DCAP_CMD_TRUNC ? node->ipc : node->pnfsId );
-
+	                                          node->asciiCommand == DCAP_CMD_TRUNC ? node->ipc : node->pnfsId );
+	
+	if (outStr == NULL){
+		goto out_of_mem_exit;
+	}
 	switch( node->asciiCommand ) {
 		case DCAP_CMD_OPEN:
 		case DCAP_CMD_TRUNC:
+#ifdef WIN32
+            if ((node->flags == O_RDONLY) || (node->flags & O_WRONLY) || (node->flags & O_RDWR) || (node->flags == O_BINARY)){
+#else
+            if ((node->flags == O_RDONLY) || (node->flags & O_WRONLY) || (node->flags & O_RDWR)	){
+#endif /* WIN32 */
+                invalid_flag = 0;
+                if ((node->file_name != NULL) && (name_invalid(node->file_name))) {
+                    dc_debug(DC_ERROR, "File '%s' contains a currently invalid dcap protocol character." , node->file_name);
+                    goto parser_exit;
+                }
+                if ((node->directory != NULL ) && (name_invalid(node->directory))) {
+                    dc_debug(DC_ERROR, "Directory '%s' contains a currently invalid dcap protocol character." , node->directory);
+                    goto parser_exit;
+                }
+            }
 #ifdef WIN32
 			if( (node->flags == O_RDONLY) || (node->flags == O_BINARY)) {
 #else
 			if( node->flags == O_RDONLY) {
 #endif /* WIN32 */
 
-				sprintf(openStr, "%s r", openStr);
-				invalid_flag = 0;
+				outStr = dc_char_buf_sprintf(context,
+					              "%s r", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 				if( (node->url == NULL) && (node->directory != NULL ) && (node->file_name != NULL) ) {
-					sprintf(openStr, "%s -path=%s/%s", openStr, node->directory, node->file_name );
+					outStr = dc_char_buf_sprintf(context,
+					              "%s -path=%s/%s", outStr, node->directory, node->file_name);
+					if (outStr == NULL) {
+						goto out_of_mem_exit;
+					}
 				}
 			}
 
 			if (node->flags & O_WRONLY) {
-				sprintf(openStr, "%s w", openStr);
-				invalid_flag = 0;
+				outStr = dc_char_buf_sprintf(context,
+					              "%s w", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 				if( (node->url == NULL) && (node->directory != NULL ) && (node->file_name != NULL) ) {
-					sprintf(openStr, "%s -path=%s/%s", openStr, node->directory, node->file_name );
+					outStr = dc_char_buf_sprintf(context,
+					              "%s -path=%s/%s", outStr, node->directory, node->file_name);
+					if (outStr == NULL) {
+						goto out_of_mem_exit;
+					}
 				}
 			}
 
 			if (node->flags & O_RDWR) {
-				sprintf(openStr, "%s rw", openStr);
-				invalid_flag = 0;
+				outStr = dc_char_buf_sprintf(context,
+					              "%s rw", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 				if( (node->url == NULL) && (node->directory != NULL ) && (node->file_name != NULL) ) {
-					sprintf(openStr, "%s -path=%s/%s", openStr, node->directory, node->file_name );
+					outStr = dc_char_buf_sprintf(context,
+					              "%s -path=%s/%s", outStr, node->directory, node->file_name);
+					if (outStr == NULL) {
+						goto out_of_mem_exit;
+					}
 				}
 			}
 
@@ -928,54 +970,109 @@ ascii_open_conversation(struct vsp_node * node)
 			 *  send file permissions in case of create with URL syntax
 			 */
 			if( (node->flags & O_CREAT) && (node->url != NULL) ) {
-				sprintf(openStr, "%s -mode=0%o", openStr, node->mode);
+				outStr = dc_char_buf_sprintf(context,
+				             "%s -mode=0%o", outStr, node->mode);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 			}
 
 			if ( node->asciiCommand == DCAP_CMD_TRUNC ) {
 				if( node->url == NULL ) {
-					sprintf(openStr, "%s -truncate=\"%s\"", openStr, node->pnfsId);
+					outStr = dc_char_buf_sprintf(context,
+						     "%s -truncate=\"%s\"", outStr, node->pnfsId);
 				}else{
-					sprintf(openStr, "%s -truncate", openStr);
+					outStr = dc_char_buf_sprintf(context,
+						     "%s -truncate", outStr);
+				}
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
 				}
 			}
-
-			sprintf(openStr, "%s %s %u", openStr, hostName, node->data_port);
-			sprintf(openStr, "%s -timeout=%ld", openStr, openTimeOut);
-
-			if(onError != onErrorDefault) {
-				sprintf(openStr, "%s -onerror=%s", openStr, onError == onErrorFail? "fail" : "retry");
-			}else{
-				sprintf(openStr, "%s -onerror=%s", openStr, "default");
+			outStr = dc_char_buf_sprintf(context,
+				"%s %s %u -timeout=%ld",
+					outStr, hostName, node->data_port, openTimeOut);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
+			switch( onError) {
+			case onErrorFail:
+				outStr = dc_char_buf_sprintf(context,
+					"%s -onerror=fail", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
+				break;
+			case onErrorDefault:
+				outStr = dc_char_buf_sprintf(context,
+					"%s -onerror=default", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
+				break;
+			default:
+			case onErrorRetry:
+				outStr = dc_char_buf_sprintf(context,
+					"%s -onerror=retry", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
+				break;
 			}
 
 			if( rqReceiveBuffer != 0 ) {
-				sprintf(openStr, "%s -send=%d", openStr, rqReceiveBuffer );
+				outStr = dc_char_buf_sprintf(context,
+					"%s -send=%d", outStr, rqReceiveBuffer);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 			}
 
 			if( rqSendBuffer!= 0 ) {
-				sprintf(openStr, "%s -receive=%d", openStr,  rqSendBuffer);
+				outStr = dc_char_buf_sprintf(context,
+					"%s -receive=%d", outStr, rqSendBuffer);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 			}
 
 
 			if( isActive() ) {
-				sprintf(openStr, "%s -passive", openStr);
+				outStr = dc_char_buf_sprintf(context,
+					"%s -passive", outStr);
+				if (outStr == NULL) {
+					goto out_of_mem_exit;
+				}
 			}
 
 			break;
 		case DCAP_CMD_STAGE:
 		case DCAP_CMD_CHECK:
-			sprintf(openStr, "%s -stagetime=%ld", openStr, node->atime);
-			sprintf(openStr, "%s -location=%s", openStr,
-				node->stagelocation == NULL ? hostName : node->stagelocation);
+			outStr = dc_char_buf_sprintf(context,
+				"%s -stagetime=%ld -location=%s",
+					outStr,
+					node->atime,
+					node->stagelocation == NULL ? hostName : node->stagelocation);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
 			invalid_flag = 0;
 			break;
 		case DCAP_CMD_MKDIR:
 		case DCAP_CMD_CHMOD:
-			sprintf(openStr, "%s -mode=%d", openStr, node->mode);
+			outStr = dc_char_buf_sprintf(context,
+				"%s -mode=%d", outStr, node->mode);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
 			invalid_flag = 0;
 			break;
 		case DCAP_CMD_CHOWN:
-			sprintf(openStr, "%s -owner=%d:%d", openStr, node->uid, node->gid);
+			outStr = dc_char_buf_sprintf(context,
+				"%s -owner=%d:%d", outStr, node->uid, node->gid);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
 			invalid_flag = 0;
 			break;
 		case DCAP_CMD_STAT:
@@ -986,11 +1083,19 @@ ascii_open_conversation(struct vsp_node * node)
 			invalid_flag = 0;
 			break;
 		case DCAP_CMD_OPENDIR:
-			sprintf(openStr, "%s %s %u", openStr, hostName, node->data_port);
+			outStr = dc_char_buf_sprintf(context,
+				"%s %s %u", outStr, hostName, node->data_port);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
 			invalid_flag = 0;
 			break;
 		case DCAP_CMD_RENAME:
-			sprintf(openStr, "%s %s", openStr, node->ipc);
+			outStr = dc_char_buf_sprintf(context,
+				"%s %s", outStr, node->ipc);
+			if (outStr == NULL) {
+				goto out_of_mem_exit;
+			}
 			invalid_flag = 0;
 			break;
 		default:
@@ -999,22 +1104,29 @@ ascii_open_conversation(struct vsp_node * node)
 	}
 
 	if (invalid_flag) {
+		dc_char_buf_free(context);
 		dc_errno = DEFLAGS;
-		free(openStr);
 		return -1;
 	}
 
 	if(extraOption != NULL) {
-		sprintf(openStr, "%s %s", openStr, extraOption);
+		outStr = dc_char_buf_sprintf(context,
+				"%s %s", outStr, extraOption);
 		free(extraOption);
 		extraOption = NULL;
+		if (outStr == NULL) {
+			goto out_of_mem_exit;
+		}
 	}
-
-	len = sprintf(openStr, "%s -uid=%d\n", openStr, uid);
-
-	sendControlMessage(node->fd, openStr, len, node->tunnel);
+	outStr = dc_char_buf_sprintf(context,
+				"%s -uid=%d\n", outStr, uid);
+	if (outStr == NULL) {
+		goto out_of_mem_exit;
+	}
+	len = strlen(outStr);
+	sendControlMessage(node->fd, outStr, len, node->tunnel);
 	/* getControlMessage(MAYBE, NULL); */
-	free(openStr);
+	dc_char_buf_free(context);
 
 	if ( (node->asciiCommand == DCAP_CMD_OPEN ) ||
 		(node->asciiCommand == DCAP_CMD_OPENDIR ) ||
@@ -1053,7 +1165,15 @@ ascii_open_conversation(struct vsp_node * node)
 	}
 
 	return 0;
-
+out_of_mem_exit:
+	dc_char_buf_free(context);
+	dc_errno = DEMALLOC;
+	return -1;
+parser_exit:
+	dc_char_buf_free(context);
+	dc_errno = DEPARSER;
+	errno = EINVAL;
+	return -1;
 }
 
 int
@@ -1080,7 +1200,7 @@ int sendControlMessage(int to, const char *buff, size_t len, ioTunnel *en)
 		m_lock(&bindLock);
 		debugMessage = xstrndup(buff, len);
 		debugMessage[len-1] = '\0';
-		dc_debug(DC_INFO, "Sending control message: %s", debugMessage);
+        dc_debug(DC_INFO, "Sending control message: %s (len=%d)", debugMessage, len);
 		free(debugMessage);
 		n = writen(to, buff, len, en);
 		m_unlock(&bindLock);
@@ -1195,7 +1315,9 @@ data_hello_conversation(struct vsp_node * node)
 
 		if (hostEnt != NULL) {
 			hostname = hostEnt->h_name;
-		} /* else -> address not resolved */
+		} else {
+			hostname = "UNKNOWN-HOST.invalid";
+		}
 
 	/* change send/receive buffer size prior any write/read operation */
 #ifdef  SO_RCVBUF
@@ -1377,6 +1499,7 @@ get_reply(int dataFd)
 	tmp = readn(dataFd, (char *) &acksize_net, sizeof(acksize), NULL);
 	if(tmp !=  sizeof(acksize)) {
 		dc_debug(DC_ERROR, "[%d] Failed to get reply.", dataFd);
+		memset(&reply, 0, sizeof(ConfirmationBlock));
 		reply.code = IOCMD_ERROR;
 		return reply;
 	}
@@ -1384,6 +1507,7 @@ get_reply(int dataFd)
 	acksize = ntohl(acksize_net);
 	if( acksize <=0 ) {
 		dc_debug(DC_ERROR, "[%d] He..!? reply is [0x%.8X](%d).", dataFd, acksize_net, acksize);
+		memset(&reply, 0, sizeof(ConfirmationBlock));
 		reply.code = IOCMD_ERROR;
 		return reply;
 	}
@@ -1393,6 +1517,7 @@ get_reply(int dataFd)
 
 	if (!ackinfo) {
 		dc_debug(DC_ERROR, "get_reply: Failed to allocate %d bytes.", acksize);
+		memset(&reply, 0, sizeof(ConfirmationBlock));
 		reply.code = IOCMD_ERROR;
 		return reply;
 	}
