@@ -9,31 +9,39 @@
  *   See the file COPYING.LIB
  *
  */
- 
- 
+
+
 /*
  * $Id: dcap_chmod.c,v 1.2 2004-11-01 19:33:29 tigran Exp $
  */
- 
- 
- #include "dcap_shared.h"
- 
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "dcap.h"
+#include "dcap_functions.h"
+#include "dcap_protocol.h"
+#include "dcap_mqueue.h"
+#include "dcap_url.h"
+#include "node_plays.h"
+#include "debug_level.h"
+#include "system_io.h"
 
 int dc_chmod( const char *path, mode_t mode)
  {
- 
- 	
+
+
 	dcap_url *url;
 	struct vsp_node *node;
 	int rc;
-	
+
 	url = (dcap_url *)dc_getURL(path);
  	if( url == NULL ) {
 		dc_debug(DC_INFO, "Using system native chmod for %s.", path);
 		return system_chmod(path, mode);
-	
+
 	}
- 
+
  	node = new_vsp_node( path );
 	if( node == NULL ) {
 		free(url->file);
@@ -48,7 +56,7 @@ int dc_chmod( const char *path, mode_t mode)
 		node->pnfsId = (char *)strdup(url->file);
 	}else{
 		node->pnfsId = (char *)strdup(path);
-	}	
+	}
 	node->asciiCommand = DCAP_CMD_CHMOD;
 
 	node->mode = mode;
@@ -56,10 +64,10 @@ int dc_chmod( const char *path, mode_t mode)
 
 	/* node cleanup procedure */
 	node_unplug( node );
-		
-	deleteQueue(node->queueID);
-	node_destroy(node);	
 
-	return rc;	
+	deleteQueue(node->queueID);
+	node_destroy(node);
+
+	return rc;
 
 }
